from distutils.core import setup
setup(
  name = 'telebirrweb',
  packages = ['telebirrweb'],
  version = '1.0.0',
  license='MIT',
  description = 'Python package used to integrate telebirr web API',
  author = 'Mukerem Ali',
  author_email = 'mukeremali112@gmail.com',
  url = 'https://github.com/mukerem/telebirrweb',
  download_url = 'https://github.com/mukerem/telebirrweb/archive/refs/tags/v1.0.0.tar.gz', 
  install_requires=[
      'pycryptodome',
      'requests',
      'rsa',
      'six',
    ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',    
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: 3.11',
  ],
)
